\name{postCP}
\alias{postCP}
\alias{print.postCP}
\alias{postCP.default}
\title{
  postCP
}
\description{
  The functions are used for change-point problems, after an initial set of change-points within the data has already been obtained. The function postCP obtains estimates of posterior probabilities of change-point and hidden states for each observation, and confidence intervals for the positions of the change-point.
}
\usage{
postCP(data,seg,model=1,lprob=numeric(),keep=FALSE,ci=0.9,initsegci=TRUE,nsamples=0,gen.data=FALSE,prior=0.5,prior.type="n",verbose=TRUE,debug=FALSE) 
}
\arguments{
function(data,seg,model=1,lprob=numeric(),keep=FALSE,ci=0.9,initsegci=TRUE,nsamples=0,gen.data=FALSE,prior=0.5,prior.type="n",verbose=TRUE,debug=FALSE)
\item{data}{A vector of observations to be segmented. Must have no missing values.}
\item{seg}{For J segments, a vector of J-1 initial estimates of change-point locations, corresponding to the index of the last observation of the first J-1 segments.}
\item{model}{Emission distribution of observed data, 1: Poisson (default), 2: normal.}
\item{lprob}{Instead of data, seg, and model, enter n*J matrix of log-densities for emission distribution, each row i corresponds to each observations, and each column j is the log-density of observation in row i in state j}
\item{keep}{Saves raw data, forward and backward matrices, and matrix of posterior change-point probabilities}
\item{ci}{Confidence interval displayed for each change-point location.}
\item{initsegci}{If true, estimate change-point confidence interval around initial segmentation (seg), if false estimate confidence intervals around observations with highest posterior probabilities.}
\item{nsamples}{Generated number of replicates of J-1 change-points.}
\item{gen.data}{Generate matrix of segmented data, each row corresponding to a different replicate.}
\item{prior}{Transition probabilities chosen as priors for HMM model. For homogeneous HMM, enter a scalar between 0 and 1 (default: 0.5). For heterogeneous HMM, enter a vector of probabilities, of length n for n observations or J for J hidden states.}
\item{prior.type}{Type of prior entered. For "n":homogeneous HMM (default), "o": vector of probabilities for each observation, "s": vector of probabilities for each hidden state.}
\item{verbose}{Additional screen output, default=TRUE.}
\item{debug}{Additional screen output for debugging purposes, default=FALSE.}
}

\value{
\item{model}{Emission distribution (Poisson or normal)} 
\item{n}{Length of segmented data}
\item{cp.est}{Matrix of estimated change-points, with lower and upper bounds of confidence intervals (given in column names)}
\item{means}{Mean of data in each of J segments}
\item{sds}{For normal data, pooled standard deviation}
\item{fbsample}{If nsamples>0, information for randomly generated change-points and data}
\item{fbsample$changepoints}{Matrix of generated changepoint locations of nsamples rows, each row containing J-1 changepoints}
\item{fbsample$data}{Matrix of generated data of nsamples rows, each row containing vector of n observations}
\item{data}{If keep=TRUE, the segmented data (if entered)}
\item{lprob}{If keep=TRUE, the log-densities (if entered)}
\item{lforward}{If keep=TRUE, matrix of log of forward probabilities, n rows corresponding to observations and J columns corresponding to states}
\item{lbackward}{If keep=TRUE, matrix of log of backward probabilities, n rows corresponding to observations and J columns corresponding to states}
\item{post.cp}{If keep=TRUE, matrix of posterior change-point probabilities, prob at row i column j is the probability of the jth change-point occurring at observation i}
\item{post.state}{If keep=TRUE, matrix of posterior hidden state probabilities, prob at row i column j is the probability of observation i being in state j}
\item{prior}{Transition probabilities chosen as priors for HMM mode.}
\item{prior.type}{Type of prior entered.}
}

\details{
\tabular{ll}{
Package: \tab postCP\cr
Type: \tab Package\cr
Version: \tab 1.0\cr
Date: \tab 2010-08-17\cr
License: \tab GPL (>= 2)\cr
LazyLoad: \tab yes\cr
}
Typical usage is todo!

}

\author{
Gregory Nuel and The Minh Luong

Maintainer: Who to complain to <gregory.nuel@parisdescartes.fr>
}

\references{
  Our manuscript.
}

\keyword{ package }

\seealso{
\code{\link{postCPsample}}
}
\examples{
require(postCP);
N=2000; x=c(rpois(N,2.0),rpois(2*N,2.2),rpois(N,1.9));
res=postCP(data=x,seg=c(N,3*N),keep=TRUE, ci=0.85);  # obtain 85% confidence intervals for changepoints, keep forward and backward matrices

## alternate method, use matrix of log-densities, n*J or 8000*3 matrix 
lprob.matrix=matrix(dpois(rep(x,3),res$means[rep(1:3,each=8000)],log=TRUE),ncol=3)
res1=postCP(lprob=lprob.matrix,keep=TRUE, ci=0.85) # obtain 85% confidence intervals for changepoints, keep forward and backward matrices

## Use results from forward-backward algorithm to obtain posterior probabities of being a change-point at each location
par(mfrow=c(2,2))
plot(x,main="original data");
plot(res$post.cp[,1],col="blue",t='l',lwd=2,main="change point distributions"); points(res$post.cp[,2],col="red",t="l",lwd=2)
## Use results to obtain probability of state membership at each point
marg.prob=exp(res$lbackward+res$lforward-res$lbackward[1,1]-res$lforward[1,1])
plot(marg.prob[,1],col="blue",t='l',lwd=2,main="marginal distributions"); points(marg.prob[,2],col="red",t="l",lwd=2); points(res$marginal[,3],col="green",t="l",lwd=2) 

## draw 1000 samples of change-points using results from postCP, and also a matrix with 1000 rows of generated data of length 8000
res.samples=postCPsample(res,1000,gen.data=TRUE);
postCP(data=x,seg=c(N,3*N),nsamples=1000,gen.data=TRUE) ## draw 1000 samples in same function call as segmenting data

# Use heterogeneous with transition probabilities 0.4 and 0.7 for 1 to 2 and 2 to 3 transitions, respectively, use 0.5 for transition to junk state J+1 from constrained HMM
postCP(data=x,seg=c(N,3*N),keep=TRUE, ci=0.85,prior=c(0.4,0.7,0.5),prior.type="s")

# Use posterior probability of change-points at each observation as prior transition probabilities, use 0.5 for transition to junk state from constrained HMM
postCP(data=x,seg=c(N,3*N),keep=TRUE, ci=0.85,prior=c(rowSums(res$post.cp),0.5),prior.type="o")

}


